#
# The Poll Tag
#
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

import App, Globals, OFS
import string
import time

from Persistence import Persistent
from Products.ZSQLMethods.SQL import SQL
from Globals import HTMLFile, HTML, MessageDialog, package_home
from Acquisition import Implicit
from OFS.Folder import Folder
from OFS.Image import manage_addImage, Image
from App.Extensions import getObject
from webdav.NullResource import NullResource
from ZPublisher.HTTPResponse import HTTPResponse
from ImageFile import ImageFile

from urllib import quote

def manage_addPollManagement(self, id, title='', REQUEST={}):
	""" Add a new Poll Management System """

	obj=PollManagement(id, title)
	self._setObject(id, obj)
	obj=getattr(self, id)
	if hasattr(obj, 'postInitialisation'):
		obj.postInitialisation()

	if REQUEST:
		return self.manage_main(self, REQUEST)
	return ''

manage_addPollManagementForm=HTMLFile('manage_addPollManagementForm', globals())

from Poll import manage_addPollForm, user_addPollForm, manage_addPoll, Poll

class PollManagement(Persistent, Implicit, Folder):
	""" Poll Administration System """

	meta_type="Poll Administration"

	manage_options=(
		{'label':'Manage Polls', 'action':'manage_editPollManagementForm'},
		{'label':'Configure Polls', 'action':'manage_configurePollManagementForm'},		
		{'label':'Contents', 'action':'manage_main'},
		{'label':'New Poll Question', 'action':'manage_addPollForm'},
		{'label':'About', 'action':'about'},
		)

	__ac_permission__= (
		('Add Polls', ('manage_addPollForm', 'manage_addPoll')),
		('Manage Polls', ('manage_editPollManagementForm',
						  'manage_editPoll',
						  'manage_configurePollManagementForm',
						  'manage_configurePollManagement',))
		)
		
	manage_configurePollManagementForm=HTMLFile('manage_configurePollManagementForm', globals())
	manage_editPollManagementForm=HTMLFile('manage_editPollManagementForm', globals())

	manage_addPollForm=manage_addPollForm
	user_addPollForm=user_addPollForm
	manage_addPoll=manage_addPoll
	
	one_color="#f0e040" #Headline Text Color
	two_color="#ffffff" #Table bg
	thr_color="#000000" #Body Text
	for_color="#102080" #Headling bg
	fiv_color="#002000" #Bottom table cell bg

	SWPTIME = 7
	HISTORY = 0
	CURRENT = 1
	SUGGESTED = 2
	CNOT = ("<font face=arial,helvetica size=-2>" +
			"The <a href=\"http://www.abrahamsson.org/poll/\">"+
			"RoxPoll software</a> was written by Thomas Abrahamsson."+
			"</font></font>")

	prehtml=HTMLFile('prehtml',globals())
	posthtml=HTMLFile('posthtml', globals())
	post1html=HTMLFile('post1html', globals())
	post2html=HTMLFile('post2html', globals())

	about=HTMLFile('about',globals())
	voted_today = {}
	running_poll_id=None
	daycache = 0
	lastavoid=0
	poll_index_html=HTMLFile('poll_index_html',globals())

	no_options=6
	enable_suggestions=1
	allowmorevotes=1
	started=0

	manage_addImage=manage_addImage
	def __init__(self, id, title=''):
		self.id=id
		self.title=title
		
	def postInitialisation(self):
		_path=globals()
		if _path is None:
			_path=SOFTWARE_HOME
		elif type(_path) is not type(''):
			_path=package_home(_path)
			
		self.base_url=self.absolute_url();
		
		f=open(_path+'/vote_left.gif','r')
		self.manage_addImage('LEFTGIF', f, 'Left Edge')
		f.close()
		f=open(_path+'/vote_right.gif','r')
		self.manage_addImage('RIGHTGIF', f, 'Right Edge')
		f.close()
		f=open(_path+'/vote_middle.gif', 'r')
		self.manage_addImage('MIDDLEGIF', f , 'Middle Bar')
		f.close()
		
	def index_html(self, REQUEST={}):
		""" Do stuff each time the poll is viewed """
		
		if not self.running_poll_id:
			try:
				self.rotate_schedule()
			except:
				import traceback
				traceback.print_exc()
		elif (getattr(self, self.running_poll_id).endTime < time.time()):
			try:
				self.rotate_schedule()
			except:
				import traceback
				traceback.print_exc()
		thisPoll=getattr(self,self.running_poll_id)
		if REQUEST.has_key('poll_vote_id'):
			vote_id=int(REQUEST['poll_vote_id'])
			thisPoll.vote(vote_id=vote_id,REQUEST=REQUEST)
		return thisPoll.index_html(REQUEST)

	def currentResults(self, REQUEST):
		""" Show the current results """
		thisPoll=getattr(self, self.running_poll_id)
		return thisPoll.currentResults(REQUEST)
	
	def suggestedPolls(self):
		""" Get All Polls that are Suggested """
		pollList=[]
		for x in self.objectValues('Poll'):
			if x.status==self.SUGGESTED:
				pollList.append(x)
		return pollList

	def historicPolls(self):
		""" Get All Polls that are Suggested """
		pollList=[]
		for x in self.objectValues('Poll'):
			if x.status==self.HISTORY:
				pollList.append(x)
		return pollList

	def getTime(self):
		""" Time """
		return time.time()

	def manage_configurePollManagement(self, base_url, no_options, one_color,
									   two_color, thr_color, for_color,
									   fiv_color, enable_suggestions=0,
									   REQUEST={}):
		""" Configure it """
		self.base_url=base_url
		self.no_options=no_options
		self.one_color=one_color
		self.two_color=two_color
		self.thr_color=thr_color
		self.for_color=for_color
		self.fiv_color=fiv_color
		self.enable_suggestions=enable_suggestions

		if REQUEST:
			return self.manage_configurePollManagementForm(self, REQUEST)
		return ''
				
	def manage_editPoll(self, subaction, avoiddoublereload, REQUEST={}):
		""" Edit Poll Things """
		if subaction == 'rotate_now':
		    if (self.lastavoid == (float(avoiddoublereload))):
				return MessageDialog(
					title='Double Reload',
					Message=(
					"Error, you tried to reload the page and do a "+
					"double shift. I just stopped you in doing that. "+
					"Congratulations! :-)"),
					action='manage_editPollForm')
		    else:
				self.rotate_schedule();
				self.lastavoid = float(avoiddoublereload)
		elif subaction == 'save_days':
			self.savedays(int(self.days) or self.SWPTIME)
		elif subaction == 'fix_schedule':
			for varib in REQUEST.keys():
				if(len(varib) > 7):
					if(varib[:7]=="weight_"):
						if hasattr(self, varib[7:]):
							new_score=(int(REQUEST[varib]))
							polid = varib[7:]
							if(new_score < 0):
								self.manage_delObjects([polid,])
							else:
								poll = getattr(self, polid)
								poll.ok=new_score
								poll.setStatus(self.SUGGESTED)

		self._p_changed-1

		if REQUEST:
			return self.manage_editPollManagementForm(self, REQUEST)
		return ''

	def viewHistory(self, id):
		if hasattr(self, id):
			thisPoll=getattr(self, id)
			return thisPoll.do_neat_table()
		return ''
	
	def rotate_schedule(self):
		pickedid=''
		weights=[]
		maxweight=0
		id=""
		for poll in self.objectValues('Poll'):
			if poll.status == self.SUGGESTED:
				if(poll.ok > maxweight):
					pickedid=poll.id
					maxweight=poll.ok

		if pickedid:
			poll=getattr(self, pickedid)
			if(maxweight > 0 and poll.ok > 0):
				if(self.running_poll_id):
					current_poll=getattr(self, str(self.running_poll_id))
					current_poll.setEndTime(time.time())
					current_poll.setStatus(self.HISTORY)

				self.current_poll_id = pickedid
				poll.setStartTime(time.time())
				poll.setEndTime(time.time() + (60 * 60 * 24 * self.loaddays()))
				self.running_poll_id = pickedid
				poll.setStatus(self.CURRENT)
				self.started=0
				self._p_changed=1

	def loaddays(self):
		if(not self.daycache):
			self.daycache=self.SWPTIME
			self._p_changed=1
		return(self.daycache)

	def savedays(self, nomber):
		self.daycache=nomber
		self._p_changed=1
